/*
 * File:  term_tuple_enumerator.h
 * Author:  mikolas
 * Created on:  Fri Dec 18 14:26:54 CET 2020
 * Copyright (C) 2020, Mikolas Janota
 */
#ifndef TERM_TUPLE_ENUMERATOR_H_7640
#define TERM_TUPLE_ENUMERATOR_H_7640
#include <vector>

#include "smt/smt_statistics_registry.h"
#include "theory/quantifiers/relevant_domain.h"
#include "theory/quantifiers_engine.h"

namespace CVC4 {
namespace theory {
namespace quantifiers {
class TermTupleEnumeratorInterface
{
 public:
  /** Initialize the enumerator. */
  virtual void init() = 0;
  /** Test if there are any more combinations. */
  virtual bool hasNext() = 0;
  /** Obtain the next combination, meaningful only if hasNext Returns true. */
  virtual void next(/*out*/ std::vector<Node>& terms) = 0;
  /** Record which of the terms obtained by the last call of next should not be
   * explored again. */
  virtual void failureReason(const std::vector<bool>& mask) = 0;
  virtual ~TermTupleEnumeratorInterface() = default;
};

struct TermTupleEnumeratorContext
{
  QuantifiersEngine* d_quantEngine;
  RelevantDomain* d_rd;
  bool d_fullEffort;
  bool d_increaseSum;
  bool d_isRd;
};

TermTupleEnumeratorInterface* mkTermTupleEnumerator(
    Node quantifier, const TermTupleEnumeratorContext* context);

}  // namespace quantifiers
}  // namespace theory
}  // namespace CVC4
#endif /* TERM_TUPLE_ENUMERATOR_H_7640 */
